"""
App init
"""

# Django
from django.utils.translation import gettext_lazy as _

__version__ = "3.0.1"
__title__ = "Intel Parser"
__title_translated__ = _("Intel Parser")
__verbose_name__ = "Intel Parser for Alliance Auth"

__esi_compatibility_date__ = "2025-12-16"

__package_name__ = "aa-intel-tool"
__app_name__ = "aa_intel_tool"
__app_name_verbose__ = "AA Intel Tool"

__github_url__ = f"https://github.com/ppfeufer/{__package_name__}"
